unit GX_ConvCodeLib;

// Most of this code comes from Sebastin Mayor <DelphiHelper@yahoo.com>

// Comment this out if you don't have the TClientDataSet component and
// can not compile the Code Librarian conversion code below
{$DEFINE HAVE_CLIENTDATASET}

interface

procedure ConvertCodeLibrarian(const DataDir: string);

implementation

{$IFDEF HAVE_CLIENTDATASET}
uses
  SysUtils, Classes, DB, DBClient, Dialogs, GpStructuredStorage;

const
  AttrTopic = 'Topic';
  AttrLanguage = 'Language';
  FileCDS = 'CodeDB.cds';
  FileFS = 'CodeLibrarian.fs';
  FieldParent = 'Parent';
  FieldKey = 'Key';
  FieldLanguage = 'Language';
  FieldCode = 'Code';
  FieldType = 'Type';
  FieldTopic = 'Topic';
  TypeValueSnippet = 'C';

var
  CodeDB: TClientDataSet;
  Storage: IGpStructuredStorage;

procedure AddDefaultIndexes;
begin
  with CodeDB do
  begin
    IndexDefs.Clear;
    AddIndex(FieldParent, FieldParent, []);
    AddIndex(FieldKey, FieldKey, []);
    IndexDefs.Update;
    IndexName := FieldParent;
  end;
end;

function GetCleanText(const Source: string): string;
const
  BadDelim: string = '\/';
  NewDelim: Char = '-';
var
  i: Integer;
begin
  Result := Source;
  repeat
    i := LastDelimiter(BadDelim, Result);
    if i > 0 then
      Result[i] := NewDelim;
  until i < 1;
end;

function PathForSnippet(ParentID: Integer; const Default: string): string;
var
  Bookmark: string;
  ParentVal: Integer;
  KeyVal: Integer;
begin
  Result := Default;

  Bookmark := CodeDB.Bookmark;
  try
    if CodeDB.Locate(FieldKey, ParentID, []) then
    begin
      Result := GetCleanText(CodeDB.FieldByName(FieldTopic).AsString) + '\' + Result;
      ParentVal := CodeDB.FieldByName(FieldParent).AsInteger;
      KeyVal := CodeDB.FieldByName(FieldKey).AsInteger;

      if (ParentVal <> 0) and (ParentVal <> KeyVal) then
        Result := PathForSnippet(ParentVal, Result);
    end;
  finally
    CodeDB.Bookmark := Bookmark;
  end;
end;

function ObjectExists(const FullName: string): Boolean;
begin
  Result := Storage.FileExists(FullName) or Storage.FolderExists(FullName);
end;

function CreateNode(ThePath: string; const Lang: string): Boolean;
var
  Topic, Path: string;
  i: Integer;
begin
  Result := False;

  repeat
    i := Pos('\', ThePath);
    Assert(i > 0);
    Topic := Copy(ThePath, 1, i - 1);
    Delete(ThePath, 1, i);

    Path := Path + '\' + Topic;
    if not ObjectExists(Path) then
      if ThePath <> '' then
      begin
        Storage.CreateFolder(Path);
        Storage.FileInfo[Path].Attribute[AttrTopic] := Topic;
      end
      else
      begin
        Storage.OpenFile(Path, fmCreate or fmOpenReadWrite);
        with Storage.FileInfo[Path] do
        begin
          Attribute[AttrTopic] := Topic;
          Attribute[AttrLanguage] := Lang;
        end;
        Result := True;
      end;
  until ThePath = '';
end;

procedure SetFileText(const FileName, Value: WideString);
var
  StringStream: TStringStream;
  fFile: TStream;
begin
  FFile := Storage.OpenFile(FileName, fmCreate or fmOpenReadWrite);
  Assert(Assigned(fFile), 'Can not open file :' + Filename);
  try
    StringStream := TStringStream.Create(Value);
    try
      FFile.Size := StringStream.Size;
      FFile.Position := 0;
      FFile.CopyFrom(StringStream, 0);
    finally
      FreeAndNil(StringStream);
    end;
  finally
    FreeAndNil(FFile);
  end;
end;

procedure ConvertCodeLibrarian(const DataDir: string);
var
  StorageFile: string;
  CurrentPath, Lang, Snippet: string;
begin
  CodeDB := TClientDataSet.Create(nil);
  try
    CodeDB.FileName := IncludeTrailingBackslash(DataDir) + FileCDS;
    if not FileExists(CodeDB.FileName) then
    begin
      MessageDlg(Format('The file %s could not be found to convert.', [CodeDB.FileName]), mtError, [mbOK], 0);
      Exit;
    end;
    CodeDB.Open;

    AddDefaultIndexes;
    StorageFile := IncludeTrailingBackslash(DataDir) + FileFS;

    Storage := CreateStructuredStorage;
    if SysUtils.FileExists(StorageFile) then
      Storage.Initialize(StorageFile, fmOpenReadWrite)
    else
      Storage.Initialize(StorageFile, fmCreate or fmOpenReadWrite);

    CodeDB.First;
    while not CodeDB.Eof do
    begin
      if SameText(CodeDB.FieldByName(FieldType).AsString, TypeValueSnippet) then // Skip folders
      begin
        Lang := CodeDB.FieldByName(FieldLanguage).AsString;
        Snippet := CodeDB.FieldByName(FieldCode).AsString;
        CurrentPath := GetCleanText(CodeDB.FieldByName(FieldTopic).AsString) + '\';

        CurrentPath := PathForSnippet(CodeDB.FieldByName(FieldParent).AsInteger, CurrentPath);

        if CreateNode(CurrentPath, Lang) then
        begin
          CurrentPath := '\' + ExcludeTrailingBackslash(CurrentPath);
          SetFileText(CurrentPath, Snippet);
        end;
      end;
      CodeDB.Next;
    end;
  finally
    FreeAndNil(CodeDB);
  end;
end;

{$ELSE not HAVE_CLIENTDATASET}
procedure ConvertCodeLibrarian(const DataDir: string);
begin
  // We can't convert the file, since we there if no TClientDataSet component
end;
{$ENDIF GX_ClientDataSet}

end.

