unit GX_ConvMainF;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ActnList, CheckLst;

type
  TfmConvMain = class(TForm)
    pnlButtons: TPanel;
    btnOK: TButton;
    btnCancel: TButton;
    gbxDirectory: TGroupBox;
    edtDirectory: TEdit;
    btnBrowseDirectory: TButton;
    gbxFilesToConvert: TGroupBox;
    ActionList: TActionList;
    actOK: TAction;
    actBrowseDirectory: TAction;
    clbFilesToConvert: TCheckListBox;
    procedure btnCancelClick(Sender: TObject);
    procedure actOKUpdate(Sender: TObject);
    procedure actOKExecute(Sender: TObject);
    procedure actBrowseDirectoryExecute(Sender: TObject);
  private
    function GetDataDirectory: string;
    procedure SetDataDirectory(const Value: string);
    function IsSomeFileChecked: Boolean;
  public
    constructor Create(AOwner: TComponent); override;

    property DataDirectory: string read GetDataDirectory write SetDataDirectory;
  end;

function AddTrailingPathDelimiter(const Directory: string): string;

var
  fmConvMain: TfmConvMain;

implementation

uses
  FileCtrl,
  Registry,
  GX_ConvFav,
  GX_ConvProofreader,
  GX_ConvCodeLib;

{$R *.DFM}

function GetGExperts11ConfigPath: string;
const
  Key = 'Software\Borland\Delphi\%d.0\GExperts-1.1\Misc';
var
  Reg: TRegistry;
  Path: string;
begin
  Result := '';
  Reg := TRegistry.Create(KEY_READ);
  try
    if Reg.KeyExists(Format(Key, [7])) then
      Reg.OpenKeyReadOnly(Format(Key, [7]))
    else if Reg.KeyExists(Format(Key, [6])) then
      Reg.OpenKeyReadOnly(Format(Key, [6]))
    else if Reg.KeyExists(Format(Key, [5])) then
      Reg.OpenKeyReadOnly(Format(Key, [5]))
    else
      Exit;
    if Reg.ValueExists('ConfigPath') then begin
      Path := Reg.ReadString('ConfigPath');
      if DirectoryExists(Path) then
        Result := Path;
    end;
  finally
    FreeAndNil(Reg);
  end;
end;

constructor TfmConvMain.Create(AOwner: TComponent);
var
  i: Integer;
begin
  inherited Create(AOwner);
  for i := 0 to Pred(clbFilesToConvert.Items.Count) do
    clbFilesToConvert.Checked[i] := True;
  edtDirectory.Text := GetGExperts11ConfigPath;
end;

procedure TfmConvMain.btnCancelClick(Sender: TObject);
begin
  Close;
end;

function TfmConvMain.GetDataDirectory: string;
begin
  Result := edtDirectory.Text;
end;

procedure TfmConvMain.SetDataDirectory(const Value: string);
begin
  edtDirectory.Text := Value;
end;

procedure TfmConvMain.actOKUpdate(Sender: TObject);
begin
  actOK.Enabled := (DataDirectory <> '') and IsSomeFileChecked;
end;

function AddTrailingPathDelimiter(const Directory: string): string;
begin
  if Length(Trim(Directory)) = 0 then
    Result := ''
  else
    if Directory[Length(Directory)] <> '\' then
      Result := Directory + '\'
    else
      Result := Directory;
end;

procedure TfmConvMain.actOKExecute(Sender: TObject);
resourcestring
  SDirDoesntExist = 'Directory "%s" does not exist!';
var
  DataDir: string;
begin
  if not DirectoryExists(DataDirectory) then
  begin
    MessageDlg(Format(SDirDoesntExist, [DataDirectory]), mtWarning, [mbOK], 0);
    edtDirectory.SetFocus;
    Exit;
  end;

  DataDir := AddTrailingPathDelimiter(DataDirectory);
  Assert(clbFilesToConvert.Items.Count = 3);
  Screen.Cursor := crHourGlass;
  try
    if clbFilesToConvert.Checked[0] then
      ConvertFavFiles(DataDir);
    if clbFilesToConvert.Checked[1] then
      ConvertProofreader(DataDir);
    if clbFilesToConvert.Checked[2] then
      ConvertCodeLibrarian(DataDir);
  finally
    Screen.Cursor := crDefault;
  end;
  ShowMessage('Conversion process complete.');
  Close;
end;

procedure TfmConvMain.actBrowseDirectoryExecute(Sender: TObject);
var
  Directory: string;
begin
  if SelectDirectory('Select installation directory', '', Directory) then
    DataDirectory := Directory;
end;

function TfmConvMain.IsSomeFileChecked: Boolean;
var
  i: Integer;
begin
  for i := 0 to Pred(clbFilesToConvert.Items.Count) do
    if clbFilesToConvert.Checked[i] then
    begin
      Result := True;
      Exit;
    end;
  Result := False;
end;

end.

