unit GX_ConvProofreader;

// Comment this out if you don't have the TClientDataSet component and
// can not compile the Code Proofreader conversion code below
{$DEFINE HAVE_CLIENTDATASET}

interface

procedure ConvertProofreader(const DataPath: string);

implementation

{$IFDEF HAVE_CLIENTDATASET}

uses
  SysUtils, DBClient, OmniXML, Dialogs;

const // Do not localize any of the strings in this const section:
  cXmlAttributeLanguage = 'Language';
  cXmlAttributeWhere = 'Where';
  cXmlNodeRoot = 'CodeProofreader';
  cXmlNodeRules = 'Rules';
  cXmlNodeAutoCorrect = 'AutoCorrectRules';
  cXmlNodeAutoCorrection = 'AutoCorrectRule';
  cXmlNodeReplaceWhat = 'ReplaceWhat';
  cXmlNodeReplaceWith = 'ReplaceWith';
  cXmlNodeDictionary = 'DictionaryEntries';
  cXmlNodeDictionaryEntry = 'DictionaryEntry';

  ReplacementSourceXmlStrings: array[0..6] of string = (
    'Pascal',
    'C++',
    'CPreProcessor',
    'SQL',
    'Assembler',
    'Strings',
    'Comments');

  GXWhereReplaceStrings: array[0..3] of string = (
    'Anywhere',
    'At the begining of a word',
    'At the end of a word',
    'Whole word only');

var
  XMLDoc : IXMLDocument;
  Root : IXMLElement;
  PI   : IXMLProcessingInstruction;
  Rule : IXMLElement;
  Node : IXMLElement;
  SubNode: IXMLElement;
  Child:IXMLElement;
  Repl : TClientDataSet;
  Dict : TClientDataSet;

procedure ConvertProofreader(const DataPath: string);
var
  idx : Integer;
begin
  Repl := TClientDataSet.Create(nil);
  Dict := TClientDataSet.Create(nil);
  try
    Dict.LoadFromFile(DataPath + 'Dictionary.cds');
    Repl.LoadFromFile(DataPath + 'Replacement.cds');

    XMLDoc := CreateXMLDoc;
    try
      XMLDoc.PreserveWhiteSpace := true;
      Root := XMLDoc.CreateElement(cXmlNodeRoot);
      PI := XMLDoc.CreateProcessingInstruction('xml','version="1.0" encoding="UTF-8"');
      XMLDoc.DocumentElement := Root;
      XMLDoc.InsertBefore(PI, Root);

      for idx := Low(ReplacementSourceXmlStrings) to High(ReplacementSourceXmlStrings) do
      begin
        Rule := XMLDoc.CreateElement(cXmlNodeRules);
        Rule.SetAttribute(cXmlAttributeLanguage, ReplacementSourceXmlStrings[idx]);
        Root.AppendChild(Rule);

        Repl.Filter := 'Language=' + IntToStr(idx);
        Repl.Filtered := True;
        Repl.First;
        Node := XMLDoc.CreateElement(cXmlNodeAutoCorrect);
        Rule.AppendChild(Node);

        while not Repl.Eof do
        begin
          SubNode := XMLDoc.CreateElement(cXmlNodeAutoCorrection);
          SubNode.SetAttribute(cXmlAttributeWhere, GXWhereReplaceStrings[Repl.FieldByName('WhereReplace').AsInteger]);
          Node.AppendChild(SubNode);
          Child := XMLDoc.CreateElement(cXMLNodeReplaceWhat);
          Child.Text := Repl.FieldByName('Typed').AsString;
          SubNode.AppendChild(Child);
          Child := XMLDoc.CreateElement(cXMLNodeReplaceWith);
          Child.Text := Repl.FieldByName('Replace').AsString;
          SubNode.AppendChild(Child);
          Repl.Next;
        end;

        Dict.Filter := 'Language=' + IntToStr(idx);
        Dict.Filtered := True;
        Dict.First;

        Node := XMLDoc.CreateElement(cXMlNodeDictionary);
        Rule.AppendChild(Node);
        while not Dict.Eof do
        begin
          SubNode := XMLDoc.CreateElement(cXMLNodeDictionaryEntry);
          SubNode.Text := Dict.FieldByName('Word').AsString;
          Node.AppendChild(SubNode);
          Dict.Next;
        end;
      end;
      XMLDoc.Save(DataPath + 'CodeProofreader.xml', ofIndent);
    finally
      XMLDoc := nil;
    end;
  finally
    Repl.Free;
    Dict.Free;
  end;
end;

{$ELSE not HAVE_CLIENTDATASET}
procedure ConvertProofreader(const DataPath: string);
begin
  // We can't convert the file, since we there if no TClientDataSet component
end;
{$ENDIF GX_ClientDataSet}

end.
